/* Parham Data Products                                                      */
/* Expert Rules Engine                                                       */
/* expert/model/rulesDetails/compoundPromo/Cart                              */
/* Copyright 2020 Wayne Parham                                               */

package com.parhamdata.expert.model.rulesDetails.compoundPromo;

import java.util.ArrayList;
import lombok.AllArgsConstructor;
import lombok.NoArgsConstructor;
import lombok.Builder;
import lombok.Data;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class Cart {
    @Data
    public static class Item {
        String     upc;
        String     description;
        String     category;
        Float      price;
        Integer    qty;
    }

    public Integer id;
    public ArrayList<Item> items;

    public Float totalPrice() {
        Float total = (float) 0;
        for (Item item : items) {
            total += lineSubtotal(item);
        }
        return total;
    }

    public Float lineSubtotal(Item item) {
        return item.getQty() * item.getPrice();
    }

    public Integer itemCount() {
        int count = 0;
        for (Item item : items) {
            count += item.getQty();
        }
        return count;
    }

    public Integer categoryCount(String category) {
        int count = 0;
        for (Item item : items) {
            if (item.getCategory().equals(category)) {
                count += item.getQty();
            }
        }
        return count;
    }

    public boolean containsUpc(String upc) {
        for (Item item : items) {
            if (item.getUpc().equals(upc)) {
                return true;
            }
        }
        return false;
    }

    public boolean containsDescription(String description) {
        for (Item item : items) {
            if (item.getDescription().equals(description)) {
                return true;
            }
        }
        return false;
    }
}