/* Parham Data Products                                                      */
/* Expert Rules Engine                                                       */
/* main/expert/services/rulesetFunctions/RulesetFunctions                    */
/* Copyright 2020 Wayne Parham                                               */

package com.parhamdata.expert.services.rulesetFunctions;

import com.google.common.base.Enums;
import com.parhamdata.expert.model.RulesetDomain;
import com.parhamdata.expert.services.databaseService.Rule;
import com.parhamdata.expert.services.databaseService.RuleRecord;
import com.parhamdata.expert.services.databaseService.RulesRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.stream.Collectors;
import java.util.ArrayList;
import java.util.Optional;
import java.util.List;

@Service
public class RulesetFunctions {

    @Autowired
    private RulesRepository rulesRepository;

    private Integer autoIncrement(String domain) {
        if ((domain == null) || (domain.equalsIgnoreCase("EXPERT")))  { domain = ""; }
        RuleRecord lastRecord = null;
        int id = 0;
        try {
            lastRecord = rulesRepository.findFirstByDomainOrderByIdDesc(domain.toUpperCase());
            if (lastRecord != null) { id = lastRecord.getId(); }
        }
        finally { id += 1; }
        return id;
    }

    public RuleRecord save(RuleRecord rule) {
        String domain = rule.getDomain();
        if ((domain == null) || (domain.equalsIgnoreCase("EXPERT")))   { rule.setDomain(""); }
        if (rule.getId() == null)   { rule.setId(autoIncrement(domain)); }
        return rulesRepository.save(rule);
    }

    public List<RuleRecord> bulkLoad(List<RuleRecord> rules) {
        List<RuleRecord> results = new ArrayList<RuleRecord>();
        for (RuleRecord rule : rules) {
            results.add(save(rule));
        }
        return results;
    }

    public Optional<RuleRecord> getRule(String domain, Integer id) {
        if ((domain == null) || (domain.equalsIgnoreCase("EXPERT")))   { domain = ""; }
        return rulesRepository.findRuleByDomainAndId(domain, id);
    }

    public List<RuleRecord> deleteByDomainAndId(String domain, Integer id) {
        if ((domain == null) || (domain.equalsIgnoreCase("EXPERT")))   { domain = ""; }
        return rulesRepository.deleteByDomainAndId(domain, id);
    }

    public List<RuleRecord> deleteByDomain(String domain) {
        if ((domain == null) || (domain.equalsIgnoreCase("EXPERT")))   { domain = ""; }
        return rulesRepository.deleteByDomain(domain);
    }

    public void deleteAll() {
        rulesRepository.deleteAll();
    }

    public List<Rule> getAllRules() {
        return rulesRepository.findAll().stream()
                .map(
                        ruleRecord -> mapFromRule(ruleRecord)
                )
                .collect(Collectors.toList());
    }

    public List<Rule> getAllRulesInDomain(String domain) {
        if ((domain == null) || (domain.equalsIgnoreCase("EXPERT")))   { domain = ""; }
        return rulesRepository.findRulesByDomain(domain).stream()
                .map(
                        ruleRecord -> mapFromRule(ruleRecord)
                )
                .collect(Collectors.toList());
    }

    public List<Rule> getRuleById(Integer id) {
        return rulesRepository.findRuleById(id).stream()
                .map(
                        ruleRecord -> mapFromRule(ruleRecord)
                )
                .collect(Collectors.toList());
    }

    private Rule mapFromRule(RuleRecord ruleRecord) {
        RulesetDomain rulesetDomain = Enums.getIfPresent(RulesetDomain.class, ruleRecord.getDomain().toUpperCase())
                .or(RulesetDomain.EXPERT);
        if (ruleRecord.getPriority() == null)   { ruleRecord.setPriority(100); }
        return Rule.builder()
                .domain(rulesetDomain)
                .id(ruleRecord.getId())
                .condition(ruleRecord.getCondition())
                .action(ruleRecord.getAction())
                .description(ruleRecord.getDescription())
                .priority(ruleRecord.getPriority())
                .build();
    }
}