/* Parham Data Products                                                      */
/* Expert Rules Engine                                                       */
/* test/expert/KeywordParserTest                                             */
/* Copyright 2020 Wayne Parham                                               */

package com.parhamdata.expert;

import com.parhamdata.expert.model.rulesEngine.langParser.KeywordParser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;
import org.junit.runner.RunWith;
import org.junit.Test;

import static org.assertj.core.api.Assertions.assertThat;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = ExpertApplication.class)
public class KeywordParserTest {

    @Autowired
    private KeywordParser keywordParser;

    final String expression1 = "input.salary >= 70000 && input.creditScore >= 900 && $(loan.interest) >= 8";
    final String expression2 = "$(input.salary) >= 70000 && input.creditScore >= 900 && $(loan.interest) >= 8";

    @Test
    public void verifyPatternInExpression(){
        String keyword = keywordParser.getListOfKeywords(expression1).get(0);
        assertThat(keyword).isEqualTo("$(loan.interest)");
    }

    @Test
    public void verifyNumberOfPatternFoundInExpression(){
        int numberOfPatters = keywordParser.getListOfKeywords(expression2).size();
        assertThat(numberOfPatters).isEqualTo(2);
    }

    @Test
    public void verifyExtractValue(){
        String keyword = keywordParser.getListOfKeywords(expression1).get(0);
        assertThat(keywordParser.extractKeyword(keyword)).isEqualTo("loan.interest");
    }

    @Test
    public void verifyKeywordResolverValue(){
        String keyword = keywordParser.getListOfKeywords(expression1).get(0);
        assertThat(keywordParser.getKeywordResolver(keywordParser.extractKeyword(keyword))).isEqualTo("loan");
    }

    @Test
    public void verifyKeywordValue(){
        String keyword = keywordParser.getListOfKeywords(expression1).get(0);
        assertThat(keywordParser.getKeywordValue(keywordParser.extractKeyword(keyword))).isEqualTo("interest");
    }
}