/* Parham Data Products                                                      */
/* Expert Rules Engine                                                       */
/* test/expert/RulesEngineTest                                               */
/* Copyright 2020 Wayne Parham                                               */

package com.parhamdata.expert;

import com.parhamdata.expert.services.databaseService.Rule;
import com.parhamdata.expert.services.rulesetFunctions.RulesetFunctions;
import com.parhamdata.expert.model.rulesDetails.loan.LoanDetails;
import com.parhamdata.expert.model.rulesDetails.loan.UserDetails;
import com.parhamdata.expert.model.RulesetDomain;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.assertj.core.util.Lists;
import org.junit.runners.MethodSorters;
import org.junit.runner.RunWith;
import org.junit.FixMethodOrder;
import org.junit.Before;
import org.junit.Test;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.http.MediaType;
import org.mockito.Mockito;

import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = ExpertApplication.class)
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class RulesEngineTest {

    private MockMvc mockMvc;
    @Autowired
    private WebApplicationContext webApplicationContext;
    @MockBean
    private RulesetFunctions runner;
    private ObjectMapper objectMapper;

    @Before
    public void setUp() {
        this.objectMapper = new ObjectMapper();
        this.mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).build();
        when(runner.getAllRulesInDomain(Mockito.any())).thenReturn(getListOfRules());
    }

    @Test
    public void verifyGetRules() throws Exception {
        mockMvc.perform(get("/getRules")
                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk()
        );
    }

    @Test
    public void verifyPostLoanRule01() throws Exception {
        UserDetails userDetails = UserDetails.builder()
                .firstName("John")
                .lastName("Doe")
                .age(25)
                .debtToIncome(15d)
                .creditScore(800)
                .monthlySalary(5000d)
                .requestedLoanAmount(999999d)
                .build();

        MvcResult mvcResult = mockMvc.perform(post("/loan")
                .contentType("application/json")
                .content(objectMapper.writeValueAsString(userDetails)))
                .andExpect(status().isOk()
                ).andReturn();

        String actualResponseBody = mvcResult.getResponse().getContentAsString();

        LoanDetails loanDetails = LoanDetails.builder()
                .approvalStatus(true)
                .interestRate(3.75f)
                .maximumPercentage(90f)
                .processingFee(4000d)
                .notes(null)
                .build();
        String expectedResponseBody = objectMapper.writeValueAsString(loanDetails);

        assertThat(expectedResponseBody).isEqualToIgnoringWhitespace(actualResponseBody);
    }

    @Test
    public void verifyPostLoanRule02() throws Exception {
        UserDetails userDetails = UserDetails.builder()
                .firstName("Jane")
                .lastName("Doe")
                .age(25)
                .debtToIncome(20d)
                .creditScore(700)
                .monthlySalary(4000d)
                .requestedLoanAmount(99999d)
                .build();

        MvcResult mvcResult = mockMvc.perform(post("/loan")
                .contentType("application/json")
                .content(objectMapper.writeValueAsString(userDetails)))
                .andExpect(status().isOk()
                ).andReturn();

        String actualResponseBody = mvcResult.getResponse().getContentAsString();

        LoanDetails loanDetails = LoanDetails.builder()
                .approvalStatus(true)
                .interestRate(4.25f)
                .maximumPercentage(90f)
                .processingFee(2000d)
                .notes(null)
                .build();
        String expectedResponseBody = objectMapper.writeValueAsString(loanDetails);

        assertThat(expectedResponseBody).isEqualToIgnoringWhitespace(actualResponseBody);
    }

    @Test
    public void verifyPostLoanRule03() throws Exception {
        UserDetails userDetails = UserDetails.builder()
                .firstName("Dick")
                .lastName("Doe")
                .age(25)
                .debtToIncome(25d)
                .creditScore(640)
                .monthlySalary(3000d)
                .requestedLoanAmount(9999d)
                .build();

        MvcResult mvcResult = mockMvc.perform(post("/loan")
                .contentType("application/json")
                .content(objectMapper.writeValueAsString(userDetails)))
                .andExpect(status().isOk()
                ).andReturn();

        String actualResponseBody = mvcResult.getResponse().getContentAsString();

        LoanDetails loanDetails = LoanDetails.builder()
                .approvalStatus(true)
                .interestRate(6.25f)
                .maximumPercentage(80f)
                .processingFee(1000d)
                .notes(null)
                .build();
        String expectedResponseBody = objectMapper.writeValueAsString(loanDetails);

        assertThat(expectedResponseBody).isEqualToIgnoringWhitespace(actualResponseBody);
    }

    @Test
    public void verifyPostLoanRule04() throws Exception {
        UserDetails userDetails = UserDetails.builder()
                .firstName("Baby")
                .lastName("Doe")
                .age(15)
                .debtToIncome(10d)
                .creditScore(800)
                .monthlySalary(500d)
                .requestedLoanAmount(1000d)
                .build();

        MvcResult mvcResult = mockMvc.perform(post("/loan")
                .contentType("application/json")
                .content(objectMapper.writeValueAsString(userDetails)))
                .andExpect(status().isOk()
                ).andReturn();

        String actualResponseBody = mvcResult.getResponse().getContentAsString();

        LoanDetails loanDetails = LoanDetails.builder()
                .approvalStatus(false)
                .interestRate(null)
                .maximumPercentage(null)
                .processingFee(null)
                .notes(null)
                .build();
        String expectedResponseBody = objectMapper.writeValueAsString(loanDetails);

        assertThat(expectedResponseBody).isEqualToIgnoringWhitespace(actualResponseBody);
    }

    @Test
    public void verifyPostLoanRule05() throws Exception {
        UserDetails userDetails = UserDetails.builder()
                .firstName("Dead")
                .lastName("Doe")
                .age(0)
                .debtToIncome(0d)
                .creditScore(0)
                .monthlySalary(0d)
                .requestedLoanAmount(0d)
                .build();

        MvcResult mvcResult = mockMvc.perform(post("/loan")
                .contentType("application/json")
                .content(objectMapper.writeValueAsString(userDetails)))
                .andExpect(status().isOk()
                ).andReturn();

        String actualResponseBody = mvcResult.getResponse().getContentAsString();

        LoanDetails loanDetails = LoanDetails.builder()
                .approvalStatus(null)
                .interestRate(null)
                .maximumPercentage(null)
                .processingFee(null)
                .notes("No matching rule.  Please examine input data and ruleset.")
                .build();
        String expectedResponseBody = objectMapper.writeValueAsString(loanDetails);

        assertThat(expectedResponseBody).isEqualToIgnoringWhitespace(actualResponseBody);
    }

    private List<Rule> getListOfRules(){
        Rule rule1 = Rule.builder()
                .domain(RulesetDomain.LOAN)
                .id(1)
                .condition("input.monthlySalary >= 5000 && input.creditScore >= 760 && input.debtToIncome < 20 && input.requestedLoanAmount < 1000000 && input.age >= 18")
                .action("output.setApprovalStatus(true);output.setMaximumPercentage(90);output.setInterestRate($(bank.prime_plus_half));output.setProcessingFee(4000);")
                .priority(1)
                .description("ruleset test 1")
                .build();
        Rule rule2 = Rule.builder()
                .domain(RulesetDomain.LOAN)
                .id(1)
                .condition("input.monthlySalary >= 4000 && input.creditScore >= 700 && input.debtToIncome < 30 && input.requestedLoanAmount < 250000 && input.age >= 18")
                .action("output.setApprovalStatus(true);output.setMaximumPercentage(90);output.setInterestRate($(bank.prime_plus_one));output.setProcessingFee(2000);")
                .priority(2)
                .description("ruleset test 2")
                .build();
        Rule rule3 = Rule.builder()
                .domain(RulesetDomain.LOAN)
                .id(3)
                .condition("input.monthlySalary >= 3000 && input.creditScore >= 640 && input.debtToIncome < 30 && input.requestedLoanAmount < 100000 && input.age >= 18")
                .action("output.setApprovalStatus(true);output.setMaximumPercentage(80);output.setInterestRate($(bank.prime_plus_three));output.setProcessingFee(1000);")
                .priority(3)
                .description("ruleset test 3")
                .build();
        Rule rule4 = Rule.builder()
                .domain(RulesetDomain.LOAN)
                .id(4)
                .condition("input.age > 0 && ( input.monthlySalary < 3000 || input.creditScore < 640 || input.debtToIncome >= 30 || input.requestedLoanAmount >= 1000000 || input.age < 18 )")
                .action("output.setApprovalStatus(false)")
                .priority(4)
                .description("ruleset test 4")
                .build();

        List<Rule> allRulesInDomain = Lists.newArrayList(rule1, rule2, rule3, rule4);
        return allRulesInDomain;
    }
}